@@ -230,8 +230,8 @@ contr.sum <- function (n, contrasts=TRUE, sparse=FALSE)
 contr.SAS <- function(n, contrasts = TRUE, sparse=FALSE)
  stats::contr.SAS(n, contrasts=contrasts, sparse=sparse)
 
-contr.poly <- function (n, scores = 1L:n, contrasts = TRUE, sparse = FALSE)
- stats::contr.poly(n, scores=scores, contrasts=contrasts, sparse=sparse)
+## the 'scores' argument default is delicate ...
+contr.poly <- stats::contr.poly
 
 `contrasts<-` <- function(x, how.many, value)
  stats::`contrasts<-`(x, how.many=how.many, value=value)
@@ -339,7 +339,7 @@ sparse2int <- function(X, Y, do.names = TRUE)
                 else { ## numeric X (1 "column"),  sparseMatrix Y
 		    r <- Y
                     dp <- Y@p[-1] - Y@p[-(Y@Dim[2]+1L)]
-                    stopifnot(all(dp %in% 0:1)) # just for now - drop! - FIXME
+		    ## stopifnot(all(dp %in% 0:1)) # just for now
                     ## if(nx == 1)
                     ## FIXME: similar trick would be applicable for nx > 2
                     r@x <- X[dp == 1L] * Y@x
